% Inverse problem for elastic substrate method
% Ulrich Schwarz, MPI Colloids and Interfaces, Germany,
% http://www.mpikg-golm.mpg.de/th/people/schwarz/
% written May 2000, modified Feb 2003 

%data.m reads in, plots and modifies original data
%INPUT:   lines.dat contains displacement data in form x1 y1 x2 y2
%         focals.dat contains positions of focals in form x1 y1
%OUTPUT:  newlines.dat contains displacement data in form x1 y1 x2 y2

% set displacements to cero which are smaller than delta2 ?
% This feature I used a lot in the beginning, since I figured
% that small displacements only represent noise and should not be used;
% now I think that there is no reason to exclude them, the
% method itself is able to handle the noise problem by itself
set_cero = 0;
delta2 = 0;
% remove all displacements which are closer to any focal than delta1
% This is important since we assume point-like forces which diverge at the origin;
% it can be shown that this approximates distributed force well as long as 
% the displacement is evaluated at distances to the ellipse which are larger
% than the ellipse itself
delta1 = 5;
% remove displacements which are farer away from any focal than delta3
% This is not necessary, but reasonable, since for displacements far
% way, the signal decays below the noise level 
delta3 = 1000;

% load data
load('lines.dat')
load('focals.dat')

Nfocals = size(focals,1);
Nlines = size(lines,1);

fprintf('Data loaded\n')
fprintf('%d displacements\n',Nlines)
fprintf('%d real focals\n',Nfocals)

data_plot(lines,focals)
wait

% check if any displacements are too close to focals for use of point-like Green function
% if so, delete them
% save modified data in newlines.dat
dellines = zeros(Nlines,1);
maxdispl = 0;
maxindex = 1;
for j = 1:Nlines
    r2 = lines(j,1:2);
    displ = [lines(j,3)-lines(j,1) lines(j,4)-lines(j,2)];
    displmag = norm(displ);   
    if (displmag > maxdispl)
        maxindex = j;
        maxdispl = displmag;
    end
    minl = 1000;
    for i = 1:Nfocals
        r1 = focals(i,1:2);
        dr = r2-r1;
        l = sqrt(dr*dr');
        if (l < minl)
            minl = l;
        end    
    end
    if ((minl < delta1) | (minl > delta3))
        dellines(j) = 1;
    end
end
delind = find(dellines);
lines(delind,:) = [];
Nlines = size(lines,1);

fprintf('Maximal displacement %f found at position %d.\n',maxdispl,maxindex)
fprintf('Minimum distance focals - displacements required %d pixels\n',delta1)
fprintf('Maximal distance focals - displacements allowed %d pixels\n',delta3)
fprintf('%d displacements deleted\n',size(delind,1))
fprintf('%d displacements left\n',Nlines)

% set small displacements to cero if requested
if set_cero == 1
    
    [lines,iset] = set_cero(lines,delta2);
    fprintf('Minimum displacement required %d pixels\n',delta2)
    fprintf('%d displacements set to cero\n',iset)
    fprintf('%d displacements left unchanged\n',Nlines-iset)
    
end

save 'newlines.dat' lines -ascii
fprintf('New displacement list saved in newlines.dat\n')

data_plot(lines,focals)
wait
